﻿/**
* @FileName:     Program.cs
* @Programmer	 Brian Hinds
* @Date:         20-02-2017
* @Version:      1.0.2
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Text.RegularExpressions;
using System.IO;

namespace CCSPortWizard
{
    internal class Program
    {
        private static void Main(string[] args)
        {
            /// <summary>
            /// These variables manages the parameters
            /// needed for the c to c-sharp transpose process
            /// </summary>
            string CSNamespace = "";
            string InputLines;
            int DestinationIndex;
            int SourceIndex;
            StreamReader SourceFile;
            StreamWriter DestinationFile;
            Match SyntaxMatch;
            Console.Clear();
            Console.ForegroundColor = ConsoleColor.DarkGreen;

            /// <summary>
            /// User command line argument validation condition
            /// </summary>
            if ((args.Length != 4) || (args[0] != "-i" && args[0] != "-o") || ((args[0] == "-i" && args[2] != "-o") && (args[0] == "-o" && args[2] != "-i")))
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine("[Invalid command line input]");
                Console.WriteLine("CCSPortWizard -i inputfile.c -o outputfile.cs");
                Console.ForegroundColor = ConsoleColor.White;
                return;
            }
            Console.WriteLine("[INPUT COMMAND VALID...]");

            /// <summary>
            /// Search for the output file argument index
            /// </summary>
            if (args[2] == "-o") { DestinationIndex = 3; SourceIndex = 1; Console.WriteLine("[OUTPUT COMMAND VALID...]"); }
            else { DestinationIndex = 1; SourceIndex = 3; }

            /// <summary>
            /// Extraction for filE name is done
            /// </summary>
            SyntaxMatch = Regex.Match(args[DestinationIndex], @"(\w+).cs");
            if (SyntaxMatch.Success) { CSNamespace = SyntaxMatch.Groups[1].Value; Console.WriteLine("[OUTPUT FILE VALID...]"); }
            else { Console.WriteLine(@"Specific destination filename is not a valid "".cs"" file"); }

            /// <summary>
            /// Using Streamreader to read the source file
            /// </summary>
            try
            {
                SourceFile = new StreamReader(args[SourceIndex]);
                DestinationFile = new StreamWriter(args[DestinationIndex]);

                Transposer Transpose = new Transposer(CSNamespace);

                bool SyntaxMainScan = false;

                InputLines = String.Empty;

                /// <summary>
                /// Loop through the source file line by line until it reaches the end of file
                /// </summary>
                while ((InputLines = SourceFile.ReadLine()) != null)
                {
                    InputLines.TrimEnd();

                    /// <summary>
                    /// This condition detects the main
                    /// </summary>
                    if (Transpose.SyntaxScan(ref InputLines))
                    {
                        DestinationFile.WriteLine(InputLines);
                        SyntaxMainScan = true;
                    }
                    else
                    {
                        /// <summary>
                        /// This condition adds a tab after the main has been detected
                        /// </summary>
                        if (SyntaxMainScan) { DestinationFile.WriteLine("\t\t" + InputLines); }
                        else { DestinationFile.WriteLine(InputLines); }
                    }
                    InputLines = String.Empty;
                }

                /// <summary>
                /// This codition will apply the curl brackets if main is found
                /// </summary>
                if (SyntaxMainScan)
                {
                    DestinationFile.WriteLine("\t}");
                    DestinationFile.WriteLine("}");
                }

                /// <summary>
                /// This will close the file streamer
                /// </summary>
                SourceFile.Close();
                DestinationFile.Close();
                Console.WriteLine("[TRANSPOSE COMPLETE...]");
            }
            catch (Exception ex)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine("[TRANSPOSE ERROR!]");
                Console.WriteLine(ex.Message);
            }
            Console.ForegroundColor = ConsoleColor.White;
            return;
        }
    }
}